@@ -15,7 +15,7 @@ class _MrRepoDirAction(Action):
 
     @classmethod
     def check_dir(this, adir, config_file_name, is_init):
-        apath = os.path.abspath(os.path.normpath(adir))
+        apath = os.path.normpath(adir)
         if not os.path.isdir(apath):
             raise ArgumentTypeError("ERROR: %s is not a directory." % apath)
 
@@ -42,12 +42,14 @@ class Repossesser(object):
     """
 
     def __init__(self, prog='mr_repo', args=None, execute=False, quiet=False,
-            config_file=".mr_repo.yml", repo_file='.this_repo', one_use=False):
+            config_file=".mr_repo.yml", repo_file='.this_repo', one_use=False,
+            verbose=False):
         self.config = {'repos': {}}
         self.repos = []
         self._command_term = 'command'
         self._config_file_name = config_file
         self._repo_file_name = repo_file
+        self.verbose = verbose
 
         # Setup parser
         self.parser = ArgumentParser(
@@ -82,6 +84,15 @@ class Repossesser(object):
     def __setup_parser(self):
         self.parser.add_argument('--version', action='version',
                 version="Mr. Repo " + version)
+
+        self.parser._config_file_name = self._config_file_name
+        self.parser.add_argument('--verbose', '-v', dest="verbose",
+                default=False,
+                help='Run this command verbosely to show debug output.',
+                action='store_true')
+        self.parser.add_argument('--dir', '-d', dest="dir", default='.',
+                help='The Mr. Repo directory being worked on.',
+                action=_MrRepoDirAction)
         subparsers = self.parser.add_subparsers(
                 title='Commands',
                 description='Valid Mr. Repo commands:',
@@ -159,42 +170,80 @@ class Repossesser(object):
             sp.add_argument('--dir', '-d', dest="dir", default='.',
                     help='The Mr. Repo directory being worked on.',
                     action=_MrRepoDirAction)
+            sp.add_argument('--verbose', '-v', dest="verbose", default=False,
+                    help='Run this command verbosely to show debug output.',
+                    action='store_true')
 
     def __path(self, spath):
         extra = " so it cannot be added to Mr. Repo."
-        try:
-            apath = os.path.normpath(spath)
-            if not os.path.isdir(apath):
-                raise ArgumentTypeError("%s is not a directory" % spath +
-                        extra)
-            if self._get_repo(apath) == None:
-                raise ArgumentTypeError("%s is not valid repository" % spath +
-                        extra)
-        except Exception as inst:
-            print("ERROR: " + inst.message)
-
+        apath = os.path.normpath(spath)
+        if not os.path.isdir(apath):
+            raise ArgumentTypeError("%s is not a directory" % spath +
+                    extra)
+        if self._get_repo(apath) == None:
+            raise ArgumentTypeError("%s is not a valid repository" % spath +
+                    extra)
         return apath
 
     # Pseudo private functions
 
-    def _get_repo(self, apath):
+    def _debug(self, debugging_info):
+        if self.verbose:
+            print("DEBUG: " + str(debugging_info))
+
+    @classmethod
+    def _get_repo(cls, apath):
         try:
             repo = git.Repo(apath)
         except:
             return None
         return repo
 
+    @classmethod
+    def find_repos(cls, start_path, max_depth=4):
+        found_repos = []
+        if max_depth >= 0:
+            (base_path, directories, filenames) = os.walk(start_path,
+                    followlinks=True).next()
+            for directory in directories:
+                directory = os.path.join(start_path, directory)
+                directory_repo = cls._get_repo(directory)
+                if isinstance(directory_repo, git.Repo):
+                    found_repos.append(os.path.normpath(directory))
+                else:
+                    found_repos.extend(cls.find_repos(directory,
+                        max_depth - 1))
+        return found_repos
+
     # Public functions
 
+    def check_repo_name(self, name):
+        repo_name = os.path.basename(os.path.normpath(name))
+        if self.is_conrtolled_repo(repo_name):
+            return repo_name
+        else:
+            return False
+
     def setup_files(self):
-        self.config_path = os.path.join(self.args.dir, self._config_file_name)
-        self.repo_file_path = os.path.join(self.args.dir, self._repo_file_name)
-        if not os.path.isfile(self.config_path):
-            self.config_file = file(self.config_path, 'w')
-        self.config_file = file(self.config_path, 'r+')
-        if not os.path.isfile(self.repo_file_path):
-            self.repo_file = file(self.repo_file_path, 'w')
-        self.repo_file = file(self.repo_file_path, 'r+')
+        temp_config_path = os.path.join(self.args.dir, self._config_file_name)
+        temp_repo_file_path = os.path.join(self.args.dir, self._repo_file_name)
+
+        # If the config files do not exist, create them
+        if not os.path.isfile(temp_config_path):
+            open(temp_config_path, 'w')
+        if not os.path.isfile(temp_repo_file_path):
+            open(temp_repo_file_path, 'w')
+
+        # Follow the link if it exists
+        self.config_path = temp_config_path if \
+                (not os.path.islink(temp_config_path)) else \
+                os.readlink(temp_config_path)
+        self.repo_file_path = temp_repo_file_path if \
+                (not os.path.islink(temp_repo_file_path)) else \
+                os.readlink(temp_repo_file_path)
+
+        self.config_file = open(self.config_path, 'r+')
+        self.repo_file = open(self.repo_file_path, 'r+')
 
     def read_config(self, check=True):
         """Read `.mr_repo.yml` and `.this_repo` files to determine state the of
@@ -247,6 +296,8 @@ class Repossesser(object):
         try:
             self.args = self.parser.parse_args(args)
             self.is_init = self.args.command == 'init'
+            if hasattr(self.args, 'verbose'):
+                self.verbose = self.verbose or self.args.verbose
             if self.args.dir == '.':
                 self.args.dir = _MrRepoDirAction.check_dir(self.args.dir,
                         self._config_file_name, self.is_init)
@@ -293,35 +344,41 @@ class Repossesser(object):
     def add_command(self, path=None):
         """Add a definition of a local repository to the Mr. Repo
         repository."""
-        path = path or self.args.path
-        repo_name = os.path.basename(path)
+        # Path relative to CWD
+        cur_rel_path = os.path.normpath(path or self.args.path)
+        # Path relative to Mr. Repo directory
+        mr_rel_path = os.path.relpath(cur_rel_path, self.args.dir)
+
+        repo_name = os.path.basename(mr_rel_path)
+
         if not self.is_conrtolled_repo(repo_name):
-            rep = self._get_repo(path)
+            rep = self._get_repo(cur_rel_path)
             if rep != None:
-                self.repos.append(os.path.basename(path))
+                self.repos.append(os.path.basename(repo_name))
                 if len(rep.remotes) > 0:
                     repo_dict = {repo_name: {'type': 'Git',
-                        'remote': rep.remote().url}, 'path': path}
+                        'remote': rep.remote().url, 'path': mr_rel_path}}
                 else:
-                    repo_dict = {repo_name: {'type': 'Git', 'path': path}}
+                    repo_dict = {repo_name: {'type': 'Git', 'path': mr_rel_path}}
+                self._debug("Adding to config: " + str(repo_dict))
                 self.config.get('repos').update(repo_dict)
                 self.write_config()
                 result = "Successfully added '%s' to Mr. Repo." % repo_name
             else:
-                result = "ERROR: %s is not a not a supported repository." % \
-                        path
+                result = "ERROR: %s is not a supported repository." % \
+                        cur_rel_path
         else:
-            result = ("ERROR: %s already in %s (i.e. controlled by Mr. "
-                    "Repo).") % (repo_name, self._config_file_name)
+            result = ("ERROR: %s is already controlled by Mr. Repo (i.e "
+                    "it is in %s).") % (repo_name, self._config_file_name)
         return result
 
     def rm_command(self):
         """Remove a definition of a local repository from the Mr. Repo
         repository. Nothing is removed from the filesystem (use `unget` for
         that."""
-        name = self.args.name
+        name = self.check_repo_name(self.args.name)
 
-        if name in self.config['repos'].keys():
+        if name:
             self.config['repos'].pop(name)
             if name in self.repos:
                 self.repos.remove(name)
@@ -366,22 +423,26 @@ class Repossesser(object):
     def get_command(self):
         """Get a repository defined in the Mr. Repo repository, but not
         available locally."""
-        name = self.args.name
+        name = self.check_repo_name(self.args.name)
 
-        if name in self.config['repos'].keys():
-            remote = self.config['repos'][name]['remote']
-            repo_type = self.config['repos'][name]['type']
-            repo_path = self.config['repos'][name]['path']
-            if repo_type == "Git":
-                new_repo = git.Repo.clone_from(remote, repo_path or
-                        self.os.path.join(self.args.dir, name))
-                self.repos.append(name)
-                ret = "Successfully cloned '%s' into '%s'." % (name,
-                    new_repo.working_dir)
-                self.write_config()
+        if name:
+            if 'remote' in self.config['repos'][name]:
+                remote = self.config['repos'][name]['remote']
+                repo_type = self.config['repos'][name]['type']
+                repo_path = os.relpath(os.path.join(self.args.dir,
+                        self.config['repos'][name]['path'] or name))
+                if repo_type == "Git":
+                    new_repo = git.Repo.clone_from(remote, repo_path)
+                    self.repos.append(name)
+                    ret = "Successfully cloned '%s' into '%s'." % (name,
+                        new_repo.working_dir)
+                    self.write_config()
+                else:
+                    ret = "ERROR: Repositories of type '%s' are not " % \
+                            repo_type + "supported"
             else:
-                ret = "ERROR: Repositories of type '%s' are not supported " % \
-                        repo_type
+                ret = "ERROR: %s does not have an associated " % name + \
+                        "remote to repossess it from."
         else:
             ret = "ERROR: '%s' is not a Mr. Repo controlled repository." % name
 
@@ -390,11 +451,11 @@ class Repossesser(object):
     def unget_command(self):
         """Remove a repository defined in the Mr. Repo repository from the
         local system."""
-        name = self.args.name
+        name = self.check_repo_name(self.args.name)
 
-        if name in self.config['repos'].keys() and name in self.repos:
-            repo_path = self.config['repos'][name]['path'] or \
-                    os.path.join(self.args.dir, name)
+        if name and name in self.repos:
+            repo_path = os.path.relpath(os.path.join(self.args.dir,
+                    self.config['repos'][name]['path'] or name))
             repo_type = self.config['repos'][name]['type']
             ret = "Successfully removed the local copy of '%s'." % name
             # Check that it is a Git repo
@@ -420,10 +481,10 @@ class Repossesser(object):
     def update_command(self):
         """Interprets Mr. Repo controlled directory and automatically updates
         tracking files based on its findings."""
-        (base_path, directories, filenames) = os.walk(self.args.dir).next()
-        repos = [os.path.join(base_path, repo) for repo in filter(lambda x: \
-                not self.is_conrtolled_repo(x) and
-                isinstance(self._get_repo(x), git.Repo), directories)]
+        start_len = len(self.repos)
+        repos = filter(lambda x: not self.is_conrtolled_repo(x),
+                self.find_repos(self.args.dir if not hasattr(self.args, 'path')
+                    else self.args.path))
 
         if hasattr(self.args, "current") and self.args.current:
             # Unavailable means it is in the config, but not in self.repos.
@@ -432,3 +493,13 @@ class Repossesser(object):
 
         # Add all of the repos
         map(self.add_command, repos)
+        difference = len(self.repos) - start_len
+        if difference > 0:
+            success_str = "Successfully added %d new repositories." % \
+                    difference
+        elif difference < 0:
+            success_str = "Successfully removed %d repositories." % \
+                    difference
+        else:
+            success_str = "No updates made to controlled repos."
+        return success_str
